
#include <QtGui>

#include "uploadthread.h"
#include "upload.h"
#include <curl/curl.h>

uploadThread::uploadThread(QObject *parent)
    : QThread(parent)
{
    mutex.lock();
    abort = false;
    mutex.unlock();
}

uploadThread::~uploadThread()
{
    mutex.lock();
    abort = true;
    mutex.unlock();

    wait();
}
void uploadThread::setLoginData(const QString & website, const QString & username, const QString & password)
{
    this->website = website;
    this->username = username;
    this->password = password;

}

void uploadThread::setCategory(const int & category)
{
    this->category = category;

}

void uploadThread::setResize(const int & resize)
{
    this->resize = resize;
}

void uploadThread::setResizeData(const int & maxwidth, const int & maxheight)
{
    this->resize = 1;
    this->maxwidth = maxwidth;
    this->maxheight = maxheight;
}

void uploadThread::setFilelistData(QStandardItemModel & filelistmodel)
{
    this->filelistmodel = &filelistmodel;
}

void uploadThread::setImageData(QImage *img)
{

}

void uploadThread::startUpload()
{
    uploadstatus = new Upload();
    connect(uploadstatus, SIGNAL(sigcancel()), this, SLOT(cancel()));
    connect(this, SIGNAL(finished()), this, SLOT(threadend()));
    connect(this, SIGNAL(updateFilename(const QString &, const QPixmap &)), uploadstatus, SLOT(updateFilename(const QString &, const QPixmap &)));
    connect(this, SIGNAL(updateProgress(const int , const int )), uploadstatus, SLOT(updateProgress(const int, const int)));

    uploadstatus->show();
    if (!isRunning()) {
        mutex.lock();
        abort = false;
        mode = 0;
        mutex.unlock();
        start(LowPriority);
    }
}

void uploadThread::getCategories()
{
    if (!isRunning()) {
        mutex.lock();
        abort = false;
        mode = 1;
        mutex.unlock();
        start(LowPriority);
    }
}

void uploadThread::cancel()
{
    mutex.lock();
    abort = true;
    mutex.unlock();

    //wait();
}

void uploadThread::threadend()
{
    uploadstatus->deleteLater();
}

void uploadThread::run()
{
    bool error = false;
    CURL *curl;

    curl_global_init(CURL_GLOBAL_ALL);
    curl = curl_easy_init();
    if (curl) {
        //curl_easy_setopt(curl, CURLOPT_VERBOSE, 1);
		curl_easy_setopt( curl, CURLOPT_WRITEFUNCTION, handleCurlWrite );

		curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 0);
		curl_easy_setopt(curl, CURLOPT_PROGRESSFUNCTION, curlProgress );
		curl_easy_setopt(curl, CURLOPT_PROGRESSDATA, this);
        //curl_easy_setopt(curl, CURLOPT_DEBUGFUNCTION, uploadThread::my_trace);
        curl_easy_setopt(curl, CURLOPT_COOKIEFILE, "");         /* just to start the cookie engine */

//        QByteArray url(QString("%1/admin.php?gallery-uploadImages-%2").arg(website, QString::number(category)).toAscii());
//        qDebug() <<"URL "<< url<<category;
//        curl_easy_setopt(curl, CURLOPT_URL, url.data());
/////////// login
        if(!error & !abort){
            if(!login(curl)){
                error = true;
            }
        }
//////// upload files mode = 0
        if(!error & !abort & mode == 0){
            if(!processFilelist(curl)){
                error = true;
            }
        }
//////// get Categories mode = 1
        if(!error & !abort & mode == 1){
            if(!emitContentAsBA(curl)){
                error = true;
            }
        }
    }

    curl_global_cleanup();
    qDebug(" thread end");
    return;
}

bool uploadThread::login(CURL *curl)
{
    qDebug("login");

    CURLcode res;
    QBuffer buffer;

    QByteArray url(QString("%1/admin.php").arg(website).toAscii());
    qDebug() <<"URL:"<< url;
    curl_easy_setopt(curl, CURLOPT_URL, url.data());

    buffer.open(QBuffer::ReadWrite);
    curl_easy_setopt( curl, CURLOPT_WRITEDATA, &buffer );

    QByteArray poststr(QString("name=%1&pass=%2&user_login_sub=Login").arg(username,password).toAscii());
    curl_easy_setopt(curl, CURLOPT_POSTFIELDS, poststr.data());
    res = curl_easy_perform(curl);
    if (res != CURLE_OK) {
        QMessageBox::information(0,"curl", curl_easy_strerror(res));
        return false;
    }

    if(QString(buffer.data()).indexOf("<title>ilch AdminMenu</title>") >= 0){
        return true;
    }else{
        QMessageBox::information(0,"login error",tr("could not log in"));
        return false;
    }
}

bool uploadThread::emitContentAsBA(CURL *curl)
{
    qDebug("emitContentAsBA");

    CURLcode res;
    QBuffer buffer;

    QByteArray url(QString("%1/admin.php?gallery").arg(website).toAscii());
    qDebug() <<"URL:"<< url;
    curl_easy_setopt(curl, CURLOPT_URL, url.data());

    buffer.open(QBuffer::ReadWrite);
    curl_easy_setopt( curl, CURLOPT_WRITEDATA, &buffer );

    res = curl_easy_perform(curl);
    if (res != CURLE_OK) {
        QMessageBox::information(0,"curl", curl_easy_strerror(res));
        return false;
    }
    if(QString(buffer.data()).indexOf("?gallery-S") >= 0){
        emit updateCategories(buffer.data());
        return true;
    }else{
        QMessageBox::information(0,"error",tr("no Categories found"));
        return false;
    }
}


bool uploadThread::processFilelist(CURL *curl)
{
    qDebug("processFilelist");

    QByteArray url(QString("%1/admin.php?gallery-uploadImages-%2").arg(website, QString::number(category)).toAscii());
    qDebug() <<"URL:"<< url;
    curl_easy_setopt(curl, CURLOPT_URL, url.data());

    QString nicename, filename, comment, ext;
    int x;
    QStandardItem *root = filelistmodel->invisibleRootItem ();
    int rc = root->rowCount();

    for(x=0; x < rc; x++){
        if(abort){
            break;
        }
        nicename = root->child(x,0)->text();
        filename = root->child(x,2)->text();
        comment = root->child(x,1)->text();
        ext = filename;
        int pos = ext.lastIndexOf(".");
        if (pos != -1)
                ext.remove(0, pos + 1);


        qDebug() << filename;
        if( filename.isEmpty() ){
            qDebug() << "clipboard";
            QImage img = qvariant_cast<QImage> (filelistmodel->index(x, 3).data());
            emit updateFilename(nicename, QPixmap().fromImage(img).scaled(100,100, Qt::KeepAspectRatio));
            //qDebug() << "blblblblbl" <<img.save("t.jpg","JPG");
            QByteArray *imgBA = new QByteArray();
                if ( resize ){
                    if(resizeImage(img, imgBA)){
                        uploadGallery(curl, nicename, comment, imgBA);
                    }
                }else{
                    if(saveImage(img, imgBA)){
                        uploadGallery(curl, nicename, comment, imgBA);
                    }
                }
            delete imgBA;
        }else{
             emit updateFilename(nicename, QPixmap(filename).scaled(100,100, Qt::KeepAspectRatio));
            if ( resize | filename.endsWith("svg") ){
                QByteArray *imgBA = new QByteArray();
                if(resizeImage(filename, imgBA)){
                    uploadGallery(curl, nicename, comment, imgBA);
                }
                delete imgBA;
            }else{
                uploadGallery(curl, nicename, comment);
            }
        }
    }
    return true;
}

bool uploadThread::uploadGallery(CURL *curl, QString &filename, QString &comment, QByteArray *imgBA)
{
    qDebug("uploadGallery");
    CURLcode res;
    //QBuffer imageBuffer = &fBuffer;
    struct curl_httppost *formpost=NULL;
    struct curl_httppost *lastptr=NULL;

    QBuffer buffer;
    buffer.open(QBuffer::ReadWrite);
    curl_easy_setopt( curl, CURLOPT_WRITEDATA, &buffer );
    qDebug()<< "uploadGallery:filename"<< filename;
        //from buffer

    if(!imgBA){
        curl_formadd(&formpost, &lastptr, CURLFORM_COPYNAME, "file[]", CURLFORM_FILE, QByteArray(filename.toAscii()).data(), CURLFORM_FILENAME, QByteArray(filename.toAscii()).data(), CURLFORM_END);
    }else{
        curl_formadd(&formpost, &lastptr, CURLFORM_COPYNAME, "file[]", CURLFORM_BUFFER, QByteArray(filename.toAscii()).data(), CURLFORM_BUFFERPTR, imgBA->data(),
                        CURLFORM_BUFFERLENGTH, imgBA->size(), CURLFORM_FILENAME, QByteArray(filename.toAscii()).data(), CURLFORM_END);
    }

    curl_formadd(&formpost, &lastptr, CURLFORM_COPYNAME, "besch[]", CURLFORM_COPYCONTENTS, QByteArray(comment.toAscii()).data(), CURLFORM_END);
    curl_formadd(&formpost, &lastptr, CURLFORM_COPYNAME, "hochladen", CURLFORM_COPYCONTENTS, "yes", CURLFORM_END);
    curl_formadd(&formpost, &lastptr, CURLFORM_COPYNAME, "submit", CURLFORM_COPYCONTENTS, "Absenden", CURLFORM_END);

    curl_easy_setopt(curl, CURLOPT_HTTPPOST, formpost);
    res = curl_easy_perform(curl);
    if (res != CURLE_OK) {
        QMessageBox::information(0,"curl", curl_easy_strerror(res));
        return false;
    }

    if(QString(buffer.data()).indexOf("erfolgreich hochgeladen") >= 0){
        return true;
    }else{
        QMessageBox::information(0,"file upload error",tr("could not upload file"));
        return false;
    }
}

bool uploadThread::resizeImage(QString &filename, QByteArray *imgBA)
{
    qDebug("resizeImage");
    QImage img(filename);
    if(img.width() == 0){

        qDebug()<< filename <<": image size " << img.width();
        return false;
    }

    if((img.width() > maxwidth) | (img.height() > maxheight)){
        img = img.scaled(maxwidth, maxheight, Qt::KeepAspectRatio, Qt::SmoothTransformation);
    }
    QBuffer imgBuf(imgBA);
    imgBuf.open(QIODevice::WriteOnly);
    if(!img.save(&imgBuf, "JPG")){
        qDebug("img could not be converted");
        return false;
    }else{
        return true;
    }
}

bool uploadThread::resizeImage(QImage &img, QByteArray *imgBA)
{
    qDebug("resizeImageFromQImage");
    if(img.width() == 0){

        qDebug() <<": image size " << img.width();
        return false;
    }
    QImage img2(img);
    if((img2.width() > maxwidth) | (img2.height() > maxheight)){
        img2 = img2.scaled(maxwidth, maxheight, Qt::KeepAspectRatio, Qt::SmoothTransformation);
    }
    QBuffer imgBuf(imgBA);
    imgBuf.open(QIODevice::WriteOnly);
    if(!img2.save(&imgBuf, "JPG")){
        qDebug("img could not be converted");
        return false;
    }else{
        return true;
    }
}

bool uploadThread::saveImage(QImage &img, QByteArray *imgBA)
{
    qDebug("saveImage");

    QBuffer imgBuf(imgBA);
    imgBuf.open(QIODevice::WriteOnly);
    if(!img.save(&imgBuf, "JPG")){
        qDebug("img could not be converted");
        return false;
    }else{
        return true;
    }
}
////////////////////////////////////////////////////////////////////////////////////


size_t uploadThread::handleCurlWrite( void *ptr, size_t size, size_t nmemb, QBuffer *buffer )
{
	qint64 s = size*nmemb;
	buffer->write( (char *) ptr, s );
	return size*nmemb;
}

int uploadThread::my_trace(CURL *handle, curl_infotype type, unsigned char *data, size_t size, void *userp)
{
  (void)handle; /* prevent compiler warning */

//QMessageBox::information(0, tr("debug"), QString( (char *) data) );
        QFile file("debug.txt");
        file.open(QIODevice::Append);
        file.write((char *) data, (qint64)size);        // write to stderr
        file.close();

  return 0;
}

int uploadThread::curlProgress (void *p, double dltotal, double dlnow, double ultotal, double ulnow)
{
    uploadThread *obj = (uploadThread *)p;
    if(ultotal > 0){
        qDebug() <<  ultotal<<ulnow;
        //emit updateFilename(nicename, filename);
        emit obj->updateProgress(int(ultotal), int(ulnow));
    }
  return 0;
}
//        res = curl_easy_perform(curl);
//        if (res != CURLE_OK) {
//            QMessageBox::information(0,"curl", curl_easy_strerror(res));
//            error= true;
//        }
//
//        if(QString(buffer->data()).indexOf("index.php?user-login") >= 0){
//            //QMessageBox::information(0,tr("ilch"),tr("found"));
//        }else{
//            QMessageBox::information(0,tr("url error"),tr("no ilch CMS found"));
//            error= true;
//        }
//        buffer->buffer().clear();
//        buffer->seek(0);












//        QStandardItem *root = filelistmodel->invisibleRootItem ();
//        int rc = root->rowCount();
//        int x;
//        struct curl_httppost *formpost=NULL;
//        struct curl_httppost *lastptr=NULL;
//        for(x=0; x < rc; x++){
//
//            emit updateProgress(root->child(x,0)->text(), root->child(x,2)->text());
//
//
//            QByteArray *imgBA = new QByteArray();
//            QBuffer *imgBuf= new QBuffer(imgBA);
//            imgBuf->open(QIODevice::WriteOnly);
//
//            QImage *img, *img2;
//
//            img = new QImage(root->child(x,2)->text());
//            //QClipboard *clipboard = QApplication::clipboard();
//            //*img = clipboard->image();
//            QMessageBox::information(0, tr("width"), QString::number(img->width()) );
//            img2 =  new QImage(img->scaled(150,150, Qt::KeepAspectRatio, Qt::SmoothTransformation));
//            //img2->save("t.png", "PNG");
//
//            if(!img2->save(imgBuf, "JPG")){
//                QMessageBox::information(0, tr("save"), "save failed" );
//            }
//            imgBuf->close();
//
//            delete img;
//            delete img2;
//            delete imgBuf;
//
//            formpost=NULL;
//            lastptr=NULL;
//
//            curl_formadd(&formpost, &lastptr, CURLFORM_COPYNAME, "file[]", CURLFORM_BUFFER, QByteArray(root->child(x,0)->text().toAscii()).data(), CURLFORM_BUFFERPTR, imgBA->data(),
//                       CURLFORM_BUFFERLENGTH, imgBA->size(), CURLFORM_FILENAME, QByteArray(root->child(x,0)->text().toAscii()).data(), CURLFORM_END);
//
//            //curl_formadd(&formpost, &lastptr, CURLFORM_COPYNAME, "file[]", CURLFORM_FILE, argv[c], CURLFORM_FILENAME, argv[c], CURLFORM_END);
//            curl_formadd(&formpost, &lastptr, CURLFORM_COPYNAME, "besch[]", CURLFORM_COPYCONTENTS, QByteArray(root->child(x,1)->text().toAscii()).data(), CURLFORM_END);
//            curl_formadd(&formpost, &lastptr, CURLFORM_COPYNAME, "hochladen", CURLFORM_COPYCONTENTS, "yes", CURLFORM_END);
//            curl_formadd(&formpost, &lastptr, CURLFORM_COPYNAME, "submit", CURLFORM_COPYCONTENTS, "Absenden", CURLFORM_END);
//
//            curl_easy_setopt(curl, CURLOPT_HTTPPOST, formpost);
//
//            res = curl_easy_perform(curl);
//            if (res != CURLE_OK) {
//                QMessageBox::information(0,"curl", curl_easy_strerror(res));
//            }
//
//            if(QString(buffer->data()).indexOf("erfolgreich hochgeladen") >= 0){
//
//                QMessageBox::information(0,tr("Upload"),tr("upload complete"));
//            }else{
//                QMessageBox::information(0,tr("Upload"),tr("upload failed"));
//            }
//            delete imgBA;
//            //QMessageBox::information(0, tr("Systray"), item->text()  );
//            if(abort){
//
//            }
//        }
