#ifndef UPLOADTHREAD_H
#define UPLOADTHREAD_H

#include <QByteArray>
#include <QMutex>
//#include <QSize>
#include <QThread>
//#include <QWaitCondition>
#include <QtGui>

#include "upload.h"
#include <curl/curl.h>
//class QImage;

class uploadThread : public QThread
{
    Q_OBJECT

public:
    uploadThread(QObject *parent = 0);
    ~uploadThread();
    static size_t handleCurlWrite(void *ptr, size_t size, size_t nmemb, QBuffer *buffer);
    static int my_trace (CURL *handle, curl_infotype type, unsigned char *data, size_t size, void *userp);
    static int curlProgress (void *p, double dltotal, double dlnow, double ult, double uln);

signals:
    void updateFilename(const QString & nicename, const QPixmap & pixmap);
    void updateProgress(const int ultotal, const int ulnow);
    void updateCategories(const QByteArray &ba);

private slots:
    void cancel();
    void setLoginData(const QString & website, const QString & username, const QString & password);
    void setCategory(const int & category);
    void setResize(const int & resize);
    void setResizeData(const int & maxwidth, const int & maxheight);
    void setFilelistData(QStandardItemModel & filelistmodel);
    void setImageData(QImage *img);
    void startUpload();

    void getCategories();
    void threadend();

protected:
    void run();
    bool login(CURL *curl);
    bool processFilelist(CURL *curl);
    bool uploadGallery(CURL *curl, QString &filename, QString &comment, QByteArray *imgBA = NULL);
    bool resizeImage(QString &filename, QByteArray *imgBA);
    bool resizeImage(QImage &img, QByteArray *imgBA);
    bool saveImage(QImage &img, QByteArray *imgBA);
    bool emitContentAsBA(CURL *curl);
//bool test(CURL *curl, QString &filename, QString &comment, QByteArray *imgBA = NULL);

private:
    QMutex mutex;
    bool abort;
    Upload *uploadstatus;
    QString website;
    QString username;
    QString password;
    int category;
    int resize;
    int maxwidth;
    int maxheight;
    QStandardItemModel *filelistmodel;
    int mode;
};

#endif
